#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const promises_1 = __importDefault(require("fs/promises"));
const compiler_1 = require("../compiler");
const parse_1 = require("../interpreter/parse");
const DirectInterpreter_1 = require("../interpreter/DirectInterpreter");
const optimize_1 = require("../interpreter/optimize");
const CTranslator_1 = require("../interpreter/CTranslator");
const program = new commander_1.Command();
const handle = (handler) => async (path, options) => {
    const content = await promises_1.default.readFile(path).then((c) => c.toString());
    const parsed = parse_1.parse(options.compiler ? compiler_1.compile(compiler_1.parse(content)).code : content);
    if (options.optimizer) {
        optimize_1.optimize(parsed);
    }
    if (options.benchmark)
        console.time("benchmark");
    const result = handler(parsed, options);
    if (options.benchmark)
        console.timeEnd("benchmark");
    if (options.out) {
        promises_1.default.writeFile(options.out, result);
    }
    else {
        console.log(result);
    }
};
program
    .command("compile <path>")
    .option("-o, --out <outpath>", "output file path")
    .action(async (path, { out }) => {
    const content = await promises_1.default.readFile(path).then((c) => c.toString());
    const compiled = compiler_1.compile(compiler_1.parse(content)).code;
    if (out) {
        promises_1.default.writeFile(out, compiled);
    }
    else {
        console.log(compiled);
    }
});
program
    .command("run <path>")
    .option("-i, --input <input...>", "input characters")
    .option("-r, --no-compiler", "run raw brainfuck code")
    .option("-s, --no-optimizer", "run raw brainfuck code")
    .option("--benchmark", "show how long this action took")
    .action(handle((statements, options) => {
    new DirectInterpreter_1.DirectInterpreter(options.input?.map(parseInt)).interpret(statements);
    return "";
}));
program
    .command("translate <path>")
    .option("-o, --out <file>", "output file")
    .option("-r, --no-compiler", "run raw brainfuck code")
    .option("-s, --no-optimizer", "run raw brainfuck code")
    .option("-l, --language <language>", "the language to that the code should be translated")
    .option("--benchmark", "show how long this action took")
    .action(handle((statements, options) => {
    if (options.language == "c++") {
        return new CTranslator_1.CTranslator().translate(statements);
    }
    throw new Error("Language not supported!");
}));
program.parse(process.argv);
